#include "lnbits.hpp"
#include "lib/shared.hpp"

LNBitsFetch* LNBitsFetch::instance = nullptr;

LNBitsFetch::LNBitsFetch() : initialized(false), taskHandle(nullptr) {}

LNBitsFetch& LNBitsFetch::getInstance() {
    if (instance == nullptr) {
        instance = new LNBitsFetch();
    }
    return *instance;
}

void LNBitsFetch::setup() {
    if (initialized) return;
    
    currentCurrency = getActiveCurrencies().front();
    
    xTaskCreate(taskLNBits, "lnbitsTask", 6 * 1024, NULL, tskIDLE_PRIORITY, &taskHandle);
    initialized = true;
    
    // Fetch rates immediately when setup
    fetchRates();
}

String LNBitsFetch::buildApiUrl(const std::string& currency) {
    String instance = preferences.getString("lnbitsInstance", DEFAULT_LNBITS_INSTANCE);
    bool useHttps = preferences.getBool("lnbitsHttps", DEFAULT_LNBITS_HTTPS);
    
    String protocol = useHttps ? "https" : "http";
    return protocol + "://" + instance + "/api/v1/rate/" + String(currency.c_str());
}

void LNBitsFetch::fetchRates() {
    if (!initialized) return;
    
    std::vector<std::string> currencies = getActiveCurrencies();
    HTTPClient http;
    
    for (const std::string& currency : currencies) {
        String apiUrl = buildApiUrl(currency);

        if (debugLogEnabled()) {
            Serial.printf("Fetching LNBits rate for %s from %s\n", currency.c_str(), apiUrl.c_str());
        }
        
        http.begin(apiUrl);
        int httpCode = http.GET();
        
        if (httpCode == HTTP_CODE_OK) {
            String payload = http.getString();
            JsonDocument doc;
            DeserializationError error = deserializeJson(doc, payload);
            
            if (!error) {
                if (doc.containsKey("price")) {
                    uint price = doc["price"].as<uint>();
                    
                    if (price > 0) {
                        priceMap[currency] = price;
                        unsigned long int currentTime = esp_timer_get_time() / 1000000;
                        lastUpdateMap[currency] = currentTime;
                        
                        if (debugLogEnabled()) {
                            Serial.printf("Updated LNBits price for %s: %d\n", currency.c_str(), price);
                        }
                        
                        // Update the screen if it's currently displaying price-related information
                        if (workQueue != nullptr && (
                            ScreenHandler::getCurrentScreen() == SCREEN_BTC_TICKER ||
                            ScreenHandler::getCurrentScreen() == SCREEN_SATS_PER_CURRENCY ||
                            ScreenHandler::getCurrentScreen() == SCREEN_MARKET_CAP)) {
                            WorkItem rateUpdate = {TASK_PRICE_UPDATE, currency[0]};
                            xQueueSend(workQueue, &rateUpdate, portMAX_DELAY);
                        }
                    }
                }
            } else if (debugLogEnabled()) {
                Serial.printf("JSON parse error for %s: %s\n", currency.c_str(), error.c_str());
            }
        } else if (debugLogEnabled()) {
            Serial.printf("HTTP error for %s: %d\n", currency.c_str(), httpCode);
        }
        
        http.end();
    }
}

void LNBitsFetch::stop() {
    if (taskHandle != nullptr) {
        vTaskDelete(taskHandle);
        taskHandle = nullptr;
    }
    initialized = false;
}

void LNBitsFetch::restart() {
    stop();
    setup();
}

bool LNBitsFetch::isInitialized() const {
    return initialized;
}

TaskHandle_t LNBitsFetch::getTaskHandle() const {
    return taskHandle;
}

uint LNBitsFetch::getPrice(const std::string& currencyCode) {
    if (priceMap.find(currencyCode) == priceMap.end()) {
        return 0;
    }
    return priceMap[currencyCode];
}

void LNBitsFetch::setPrice(uint newPrice, const std::string& currencyCode) {
    priceMap[currencyCode] = newPrice;
}

unsigned long int LNBitsFetch::getLastPriceUpdate(const std::string& currencyCode) {
    if (lastUpdateMap.find(currencyCode) == lastUpdateMap.end()) {
        return 0;
    }
    return lastUpdateMap[currencyCode];
}

void setupLnbits() {
    if (preferences.getBool("lnbitsEnabled", DEFAULT_LNBITS_ENABLED)) {
        LNBitsFetch::getInstance().setup();
    }
}

void taskLNBits(void* pvParameters) {
    for (;;) {
        // Wait for notification from timer
        ulTaskNotifyTake(pdTRUE, portMAX_DELAY);
        
        // Fetch rates
        LNBitsFetch::getInstance().fetchRates();
    }
} 

void LNBitsFetch::setCurrentCurrency(const std::string& currency) {
    currentCurrency = currency;
}

std::string LNBitsFetch::getCurrentCurrency() const {
    return currentCurrency;
}